import _ from 'underscore'
import queryString from 'query-string'
import cookie from '../managers/cookie'

// import request from 'request-promise'
import axios from 'axios'

class BaseApi {

    constructor() {
        this._METHOD_GET = 'get'
        this._METHOD_POST = 'post'
        this._METHOD_PATCH = 'patch'
        this._METHOD_DELETE = 'delete'
        let extension = process.env.NODE_ENV === 'production' ? 'com/' : 'xyz/'
        this._BASE_URL = 'https://auth.marketplays.' + extension
    }

    /**
     * Get api path
     * @return {string}
     */
    getApiPath() {
    }

    /** Assign an object */
    getDataObject() {
    }

    /**
     * create response
     * @param {boolean} isSuccess
     * @param {function} response
     * @return {{success: *, data: *}}
     */
    static createResponse(isSuccess, response) {
        return {success: isSuccess, data: response}
    }

    /**
     * Get record list
     * @param {object} params
     * @return {Promise}
     */
    getRecordsList(params) {
        let additionalParams = params ? '?' + queryString.stringify(params) : ''
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_GET, _this.getApiPath() + additionalParams)
                .then(result => {
                    let newArray = []
                    if (result.total !== 0) {
                        let iThis = _this
                        _.map(result.records, function (record) {
                            newArray.push(iThis.getDataObject().parseDataFromResourceServer(record))
                        })
                        result.records = newArray
                        resolve(result)
                    } else {
                        resolve(result)
                    }
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Get single record
     * @param {number} recordId
     * @return {Promise}
     */
    getSingleRecord(recordId) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_GET, _this.buildUrlForSingleRecord(recordId), [])
                .then(result => {
                    resolve(_this.getDataObject().parseDataFromResourceServer(result.data))
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Create record
     * @param {object} data
     * @return {Promise}
     */
    createRecord(data) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_POST, _this.getApiPath(), data)
                .then(result => {
                    resolve(_this.getDataObject().parseDataFromResourceServer(result.data))
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Update record
     * @param {number} recordId
     * @param {object} data
     * @return {Promise}
     */
    updateRecord(recordId, data) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_PATCH, _this.buildUrlForSingleRecord(recordId), data)
                .then(result => {
                    resolve(_this.getDataObject().parseDataFromResourceServer(result.data))
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Delete record
     * @param {number} recordId
     * @return {Promise}
     */
    deleteRecord(recordId) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_DELETE, _this.buildUrlForSingleRecord(recordId), [])
                .then(result => {
                    resolve(result)
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Build url for single record
     * @param {number} recordId
     * @return {string}
     */
    buildUrlForSingleRecord(recordId) {
        return this.getApiPath() + '/' + recordId
    }

    /**
     * Make api request
     * @param {string} method
     * @param {string} url
     * @param {array} data
     * @return {Promise}
     */
    makeApiRequest(method, url, data) {
        let finalUrl = this._BASE_URL + 'api/' + url
        const config = {
            method: method,
            url: finalUrl,
            data: data
        }

        // global for post
        axios.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded'

        // global for every request
        axios.defaults.headers.common['HTTP_X_MP_LANGUAGE'] = 'en' // TODO: set in client
        axios.defaults.headers.common['Accept'] = 'application/json'

        // config.headers = {'Authorization': 'Bearer NDM2MjVkZjhiNTYwOWM0NDI1YWM1YWYxMWM3NjE0NjM3YTA1MDFmMTIyNDAxMjU5ODMyYzczMzFjMGQ0NDBlNw'}
        if (cookie.getCurrentToken()) {
            config.headers = {'Authorization': 'Bearer ' + cookie.getCurrentToken()}
        }
        return new Promise(function (resolve, reject) {
            axios.request(config)
                .then(function (response) {
                    resolve(response)
                })
                .catch(err => {
                    reject(err)
                })
        })
    }
}

export default BaseApi
